.setcpu "65C02"
.segment "CODE"

; okay so rn i wanna set up a very basic system init, and write a few subroutines to draw characters at x,y coordinates
    n = $01 ; temporary storage for data stack operations

.macro breakpoint ; $02 isn't a valid instruction, the emulator will see this and halt, dump memory contents
    .byte $02
.endmacro 

.macro pop ; drops a data stack cell
    inx
    inx
.endmacro

.macro pop2 ; drops 2 data stack cells
    inx
    inx
    inx
    inx
.endmacro

.macro push ; push a data stack cell
    dex
    dex
.endmacro

.macro push2 ; push 2 data stack cells
    dex
    dex
    dex
    dex
.endmacro

.macro push_char char ; pushes an ascii character code onto the stack
    lda char
    push
    sta 0, x ; char low byte
    stz 1, x ; char high byte
.endmacro

.macro push_coords coord_x, coord_y ; push a set of (x,y) coordinates onto the data stack
    lda coord_x
    push
    sta 0, x ; low byte
    stz 1,x ; high byte is zero
    lda coord_y
    push
    sta 0,x ; same here
    stz 1,x
.endmacro

.macro to_r ; pop the top of the stack off and save it in the return (hardware) stack: (n -- )
    lda 1, x
    pha
    lda 0, x
    pha
    pop
.endmacro

.macro from_r ; pop the top of the return stack off and put it on the data stack: ( -- n)
    push
    pla
    sta 0, x
    pla
    sta 1, x
.endmacro

init:
    ldx #0; initialize data stack pointer
    jsr initdisplay

main:
    jsr draw
    jmp main

initdisplay:
    lda #$20
    ldy #0
    jsr cleardisplay
    rts

cleardisplay:
    sta $6000,y
    sta $6100,y
    sta $6200,y
    sta $6300,y
    sta $6400,y
    sta $6500,y
    sta $6600,y
    sta $6700,y ; this goes slightly over but it's fine
    iny
    bne cleardisplay
    rts

draw:
    lda #%01000000
    bit $4400
    beq @1
    push_coords #0, #0
    push_char #$77
    jsr draw_char
    @1: 
    rts

draw_char: ; draw a character c at (x, y) (n1: x n2: y n3: c -- )
    lda 0, x ; load a with character to draw
    pop ; and pop it off the stack
    jsr get_char_address ; calculate where to put the character in memory
    sta (0, x) ; store a at the address pointed to on the stack
    rts

get_char_address:   ; gets vram address for a character at (x, y),
                    ; (n1: x n2: y -- n: $6000 + x + (64 * y))
    ;jsr push_lit ; push 64 onto stack, low byte first
    ;.byte 64
    ;.byte 0
    pha
    lda #64
    push ; doing this instead until `push_lit` is fixed
    sta 0, x
    stz 1, x
    jsr mult ; multiply 64 with y (n2)
    jsr plus ; add result with x (n1)

    ;jsr push_lit ; push vram address onto the stack
    ;.byte $00
    ;.byte $60
    lda #$60
    push
    sta 1, x
    stz 0, x
    jsr plus ; add vram start address to result

    pla
    rts

fill: ; fills an area from (x1, y1) to (x2, y2) will character c, (n1: c n2: x1 n3: y1 n4: x2 n5: y2 -- )
    
    jsr get_char_address

; --- Data Stack --- ;
; on this channel we love garth wilson: https://wilsonminesco.com/stacks/StackOps.ASM
; data stack is built up of 2-byte cells


; TODO: this is broken, jumping here does nothing to the stack and skips several instructions, could be an emulator problem tho
push_lit: ; this bad boy lets you inline a literal (low byte first) right after `jsr push_lit` and put it on the stack, once again, on this channel we love garth wilson
   push2
   phx
       tsx
       txa
       tay
   plx

   lda $102, y
   sta 0, x
   clc
   adc #2
   sta $102, y

   lda $103, y
   sta 1, x
   adc #0
   sta $103, y

fetch: 
    lda (0, x)
    pha
    inc 0, x
    bne @1
    inc 1, x
@1:     lda (0, x)
        bra put
    push

put:
    sta 1, x
    pla
    sta 0, x
    rts

plus: ; add: (n1 n2 -- n1+n2)
    clc
    lda 0, x
    adc 2, x
    sta 2, x
    lda 1, x
    adc 3, x
    sta 3, x
    pop
    rts


mult: ; multiply: (n1 n2 -- n1*n2), frankly, i don't know how this works, but TODO: will try to figure it out later
    phy
    stz n
    ldy #0
@1:     lsr 3, x
        ror 2, x
        bcc @2
            clc
            lda n
            adc 0, x
            sta n
            tya
            adc 1, x
            tay
@2:     asl 0, x
        rol 1, x
        lda 2, x
        ora 3, x
    bne @1
    lda n
    sta 2, x
    sty 3, x
    pop
    ply
    rts
        

.segment "VRAM"
